﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.ServiceModel.Configuration;
using System.Configuration;
using System.Collections;
using System.Text.RegularExpressions;
using System.Runtime.Serialization.Formatters.Binary;

//
// Copyright (c) 2012, TomTom 
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// * Redistributions of source code must retain the above copyright notice, this
//   list of conditions and the following disclaimer.
// * Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
// * Neither the name of the TomTom, WEBFLEET nor the names of its
//   contributors may be used to endorse or promote products derived from this
//   software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
// 

namespace ItnWaypoints
{
  public partial class MainForm : Form
  {
    readonly List<Waypoint> wpList = new List<Waypoint>();
    readonly BindingSource bindingSource = new BindingSource();
    bool mustShowCopyright = true;


    public MainForm()
    {
      InitializeComponent();
      this.waypointDataGridView.DataSource = bindingSource;
      bindingSource.DataSource = this.wpList;
      this.encodingComboBox.SelectedIndex = 0;

      FillServers();
      FillCountries();
      RestoreState();
    }

    private void MainForm_Load(object sender, EventArgs e)
    {
    }

    private void FillServers()
    {
      // Automagically find all client endpoints defined in app.config
      var clientSection = ConfigurationManager.GetSection("system.serviceModel/client") as ClientSection;
      var endpointCollection = clientSection.Endpoints;
      var endpointNames = (from ChannelEndpointElement item in endpointCollection select item.Name).ToArray();

      Dictionary<string, HostEntry> dict = new Dictionary<string, HostEntry>();
      foreach (string name in endpointNames)
      {
        string [] split = name.Split('-');
        HostEntry he;
        if (dict.ContainsKey(split[0]))
        {
          he = dict[split[0]];
        }
        else 
        {
          dict[split[0]] = he = new HostEntry { Name = split[0] };
        }
        if (split[1].Equals("orders"))
        {
          he.OrdersEndpoint = name;
        }
        else if (split[1].Equals("address"))
        {
          he.AddressEndpoint = name;
        }
      }
      var array = dict.Values.ToArray();

      this.endPointComboBox.DisplayMember = "Name";
      this.endPointComboBox.DataSource = array;

      // hide server choice box if there's only 1 endpoint configured
      if (this.endPointComboBox.Items.Count < 2)
      {
        this.endPointComboBox.Visible = false;
        this.endpointLabel.Visible = false;
      }

    }

    private void FillCountries()
    {
      // Add some convenience entries. User can type other county codes in the combo box.
      this.addrCountryComboBox.Items.Add("DE");
      this.addrCountryComboBox.Items.Add("GB");
      this.addrCountryComboBox.Items.Add("NL");
      this.addrCountryComboBox.Items.Add("FR");
      this.addrCountryComboBox.Items.Add("ES");
      this.addrCountryComboBox.Items.Add("DK");
      this.addrCountryComboBox.Items.Add("IT");
      this.addrCountryComboBox.Items.Add("PL");
      this.addrCountryComboBox.Items.Add("US");
      this.addrCountryComboBox.SelectedIndex = 0;
    }

    private HostEntry SelectedEndpoint
    {
      get
      {
        return this.endPointComboBox.Items[this.endPointComboBox.SelectedIndex] as HostEntry;
      }
    }


    private void sendOrderButton_Click(object sender, EventArgs e)
    {
      SendOrder();
    }

    /// <summary>
    /// Send a new order by using WF.connects sendDestinationOrder function.
    /// </summary>
    private void SendOrder()
    {
      // some minimal checks...
      if (this.objectNoTextBox.Text.Length == 0)
      {
        MessageBox.Show("Please enter an object number!", "Missing object number");
        return;
      }

      if (this.addrTownTextBox.Text.Length == 0 || this.addrStreetTextBox.Text.Length == 0)
      {
        MessageBox.Show("Please enter an order destination address!", "Missing order destination");
        return;
      }

      // geocode the entered address
      AddressService.CompleteLocationWithAdditionalInformation geoCodedAddr = GeocodeAddress();
      if (geoCodedAddr == null)
      {
        // geocoding not successful
        return;
      }

      var aParm = new OrdersService.AuthenticationParameters
      {
        accountName = this.accountTextBox.Text,
        userName = this.userTextBox.Text,
        password = this.passwordTextBox.Text,
        apiKey = this.apiKeyTextBox.Text.Length > 0 ? this.apiKeyTextBox.Text : null
      };
      var gParm = new OrdersService.GeneralParameters();

      // build the order parameters object
      var dstOrderParm = new OrdersService.DestinationOrderParameter();
      dstOrderParm.orderNo = this.orderNoTextBox.Text;
      dstOrderParm.orderText = this.orderTextTextBox.Text;
      dstOrderParm.explicitDestination = new OrdersService.LocationParameter()
      {
        city = geoCodedAddr.city,
        country = geoCodedAddr.countryCode,
        postcode = geoCodedAddr.postcode,
        street = geoCodedAddr.street,
        streetNumber = geoCodedAddr.streetNumber,
        geoPosition = new OrdersService.GeoPositionParameter{
          latitude = geoCodedAddr.geoPosition.latitude.GetValueOrDefault(),
          longitude = geoCodedAddr.geoPosition.longitude.GetValueOrDefault()
        }
      };

      var objParm = new OrdersService.ObjectIdentityParameter();
      objParm.objectNo = this.objectNoTextBox.Text;
        
      // add the waypoints from the list in the dialog
      var advOrderParm = new OrdersService.AdvancedSendDestinationOrderParameter();
      advOrderParm.waypoints = ReadWaypoints();

      try
      {
        var client = new OrdersService.ordersClient(SelectedEndpoint.OrdersEndpoint);
        var result = client.sendDestinationOrder(aParm, gParm, dstOrderParm, objParm, advOrderParm);
        if (result.statusCode == 0)
        {
          OrdersService.ReportedOrderData newOrder = LoadRecentOrder(objParm.objectNo);
          DisplaySentOrder(objParm.objectNo, newOrder, advOrderParm.waypoints);
        }
        else
        {
          Tools.DisplayError(result);
        }
      }
      catch (Exception e)
      {
        MessageBox.Show("Error: " + e.Message, "Error");
      }

    }

    /// <summary>
    /// Updates an existing order.
    /// </summary>
    private void UpdateOrder()
    {
      if (this.orderNoTextBox.Text.Length == 0)
      {
        MessageBox.Show("Please enter an order number!", "Missing order number");
        return;
      }

      var aParm = new OrdersService.AuthenticationParameters
      {
        accountName = this.accountTextBox.Text,
        userName = this.userTextBox.Text,
        password = this.passwordTextBox.Text,
        apiKey = this.apiKeyTextBox.Text.Length > 0 ? this.apiKeyTextBox.Text : null
      };
      var gParm = new OrdersService.GeneralParameters();

      var objParm = new OrdersService.ObjectIdentityParameter();
      objParm.objectNo = this.objectNoTextBox.Text;

      var updParam = new OrdersService.DestinationOrderUpdateParameter();
      updParam.orderNo = this.orderNoTextBox.Text;
      updParam.orderText = this.orderTextTextBox.Text;

      if (this.addrTownTextBox.Text.Length > 0 && this.addrStreetTextBox.Text.Length > 0)
      {
        AddressService.CompleteLocationWithAdditionalInformation geoCodedAddr = GeocodeAddress();
        if (geoCodedAddr == null)
        {
          // geocoding not successful
          return;
        }
        updParam.explicitDestination = new OrdersService.LocationParameter()
        {
          city = geoCodedAddr.city,
          country = geoCodedAddr.countryCode,
          postcode = geoCodedAddr.postcode,
          street = geoCodedAddr.street,
          streetNumber = geoCodedAddr.streetNumber,
          geoPosition = new OrdersService.GeoPositionParameter
          {
            latitude = geoCodedAddr.geoPosition.latitude.GetValueOrDefault(),
            longitude = geoCodedAddr.geoPosition.longitude.GetValueOrDefault()
          }
        };
      }

      var advUpdParam = new OrdersService.AdvancedUpdateDestinationOrderParameter();
      advUpdParam.waypoints = ReadWaypoints();

      try
      {

        var client = new OrdersService.ordersClient(SelectedEndpoint.OrdersEndpoint);
        var result = client.updateDestinationOrder(aParm, gParm, updParam, advUpdParam);
        if (result.statusCode == 0)
        {
          MessageBox.Show("The order was updated successfully.", "Order updated");
        }
        else
        {
          Tools.DisplayError(result);
        }
      }
      catch (Exception e)
      {
        MessageBox.Show("Error: " + e.Message, "Error");
      }
    }



    /// <summary>
    /// Converts the waypoints in the waypoints list of the dialog to the format needed by WF.connect
    /// </summary>
    /// <returns></returns>
    private OrdersService.ItnWaypointParameter[] ReadWaypoints()
    {
      if (this.waypointDataGridView.RowCount == 0)
      {
        return null;
      }
      List<OrdersService.ItnWaypointParameter> paramList = new List<OrdersService.ItnWaypointParameter>();

      foreach (Waypoint wp in this.wpList)
      {
        var param = new OrdersService.ItnWaypointParameter();
        param.latitude = wp.Latitude;
        param.longitude = wp.Longitude;
        param.description = wp.Description;
        if (param.description != null && param.description.Length > 20)
        {
          param.description = param.description.Substring(0, param.description.Length - (param.description.Length - 20));
        }
        param.notifyPassing = wp.Notify;
        param.notifyPassingSpecified = true;
        param.showInList = wp.Visible;
        param.showInListSpecified = true;
        paramList.Add(param);
      }
      return paramList.ToArray();
    }

    private void importItnButton_Click(object sender, EventArgs e)
    {
      OpenFileDialog dlg = new OpenFileDialog();
      dlg.Title = "Load waypoints from ITN file";
      dlg.DefaultExt = "itn";
      dlg.Filter = "ITN files (*.itn)|*.itn|All files (*.*)|*.*";
      if (DialogResult.OK == dlg.ShowDialog())
      {
        ImportItnFile(dlg.FileName);
      }
      dlg.Dispose();
    }


    private void ImportItnFile(string fileName)
    {
      /*
        Field 0 Longitude in WGS84 format (in 100 000th of degrees)
        Field 1 Latitude in WGS84 format (in 100 000th of degrees)
        Field 2 Name of this point in the itinerary
        Field 3 Flag
            0x00 None
            0x01 Itinerary location is enabled
            0x02 Itinerary location is a stop−over (as opposed to a
                  pass−by location)
            0x04 Itinerary location is departure point (Should only
                  be set for the first item in the itinerary file) 
       */
      this.wpList.Clear();
      StreamReader sr = new StreamReader(fileName, this.encodingComboBox.SelectedIndex == 0 ? Encoding.Default : Encoding.UTF8);
      try
      {
        string line = sr.ReadLine();
        while (line != null)
        {
          string[] fields = line.Split(new char[] { '|' });
          Waypoint wp = new Waypoint
          {
            Longitude = Int32.Parse(fields[0]) * 10,
            Latitude = Int32.Parse(fields[1]) * 10,
            Description = fields[2],
            Notify = true,
            Visible = true,
          };
          this.wpList.Add(wp);
          line = sr.ReadLine();
        }
      }
      finally
      {
        sr.Close();
      }
      this.bindingSource.ResetBindings(false);
    }

    private void clearWaypointsButton_Click(object sender, EventArgs e)
    {
      this.wpList.Clear();
      this.bindingSource.ResetBindings(false);
    }

    private void addWpButton_Click(object sender, EventArgs e)
    {
      this.wpList.Add(new Waypoint());
      this.bindingSource.ResetBindings(false);
    }

    private void delWpButton_Click(object sender, EventArgs e)
    {
      if (this.waypointDataGridView.CurrentRow != null && this.waypointDataGridView.CurrentRow.Index >= 0)
      {
        this.wpList.RemoveAt(this.waypointDataGridView.CurrentRow.Index);
        this.bindingSource.ResetBindings(false);
      }
    }

    private void waypointDataGridView_RowsRemoved(object sender, DataGridViewRowsRemovedEventArgs e)
    {
      this.waypointCountLabel.Text = "" + this.waypointDataGridView.RowCount;
    }

    private void waypointDataGridView_RowsAdded(object sender, DataGridViewRowsAddedEventArgs e)
    {
      this.waypointCountLabel.Text = "" + this.waypointDataGridView.RowCount;
    }

    private void addrCheckButton_Click(object sender, EventArgs e)
    {
      GeocodeAddress();
    }

    private AddressService.CompleteLocationWithAdditionalInformation GeocodeAddress()
    {
      var aParm = new AddressService.AuthenticationParameters
      {
        accountName = this.accountTextBox.Text,
        userName = this.userTextBox.Text,
        password = this.passwordTextBox.Text,
        apiKey = this.apiKeyTextBox.Text.Length > 0 ? this.apiKeyTextBox.Text : null
      };
      var gParm = new AddressService.GeneralParameters();

      var client = new AddressService.addressClient(SelectedEndpoint.AddressEndpoint);
      var location = new AddressService.GeocodingByProviderParameter
      {
        city = this.addrTownTextBox.Text,
        postcode = this.addrZipCodeTextBox.Text,
        street = this.addrStreetTextBox.Text,
        streetNumber = this.addrStreetNoTextBox.Text,
        countryCode = this.addrCountryComboBox.Text as string
      };

      try
      {
        var result = client.geocodeAddress(aParm, gParm, location);
        if (result.statusCode == 0)
        {
          var addr = result.results[0] as AddressService.CompleteLocationWithAdditionalInformation;
          this.addrStreetTextBox.Text = addr.street;
          this.addrTownTextBox.Text = addr.city;
          this.addrStreetNoTextBox.Text = addr.streetNumber;
          this.addrZipCodeTextBox.Text = addr.postcode;
          this.addrCountryComboBox.Text = addr.countryCode;
          this.latLonLabel.Text = addr.geoPosition.latitude + " " + addr.geoPosition.longitude;
          return addr;
        }
        else
        {
          Tools.DisplayError(result);
        }
      }
      catch (Exception e)
      {
        MessageBox.Show("Error: " + e.Message, "Error");
      }
      return null;
    }


    private void DisplaySentOrder(string objectNo, OrdersService.ReportedOrderData newOrder, OrdersService.ItnWaypointParameter[] waypoints)
    {
      OrderAckDlg dlg = new OrderAckDlg(objectNo, newOrder, waypoints);
      dlg.ShowDialog();
      dlg.Dispose();
    }


    private void addrSelectButton_Click(object sender, EventArgs e)
    {
      SelectAddress();
    }

    private void SelectAddress()
    {
      var aParm = new AddressService.AuthenticationParameters
      {
        accountName = this.accountTextBox.Text,
        userName = this.userTextBox.Text,
        password = this.passwordTextBox.Text,
        apiKey = this.apiKeyTextBox.Text.Length > 0 ? this.apiKeyTextBox.Text : null
      };
      var gParm = new AddressService.GeneralParameters();
      var client = new AddressService.addressClient(SelectedEndpoint.AddressEndpoint);

      try
      {
        var result = client.showAddressReport(aParm, gParm, null);
        if (result.statusCode == 0)
        {
          var addresses = (from AddressService.Address item in result.results select item).ToArray();

          AddressDlg dlg = new AddressDlg(addresses);
          List<AddressService.Address> list = new List<AddressService.Address>();

          if (dlg.ShowDialog() == DialogResult.OK)
          {
            AddressService.Address sel = dlg.SelectedAddress;
            if (sel != null)
            {
              this.addrCountryComboBox.Text = sel.location.country;
              this.addrTownTextBox.Text = sel.location.city;
              this.addrZipCodeTextBox.Text = sel.location.postcode;
              this.addrStreetTextBox.Text = sel.location.street;
              this.addrStreetNoTextBox.Text = "";  // unfortunately house numbers are not separately stored in WF addresses, they are part of the street name
              this.latLonLabel.Text = sel.location.geoPosition.latitude + " " + sel.location.geoPosition.longitude;

              if (sel.location.street != null)
              {
                Regex r1 = new Regex("[\\s,](\\d+)$");
                Match m = r1.Match(sel.location.street);
                if (m != null && m.Groups.Count == 2)
                {
                  this.addrStreetNoTextBox.Text = m.Groups[1].Value;
                  this.addrStreetTextBox.Text = sel.location.street.Substring(0, m.Index);
                }
              }
            }
          }
          dlg.Dispose();
        }
        else
        {
          Tools.DisplayError(result);
        }
      }
      catch (Exception e)
      {
        MessageBox.Show("Error: " + e.Message, "Error");
      }

    }

    private void addToWaypointsButton_Click(object sender, EventArgs e)
    {
      AddAddressAsWaypoint();
      ClearAddressFields();
    }

    private void ClearAddressFields()
    {
      this.addrStreetTextBox.Text = "";
      this.addrStreetNoTextBox.Text = "";
      this.addrTownTextBox.Text = "";
      this.addrCountryComboBox.SelectedIndex = 0;
      this.addrZipCodeTextBox.Text = "";
    }

    /// <summary>
    /// Geocodes the address and uses the geo coordinates to add a waypoint.
    /// </summary>
    private void AddAddressAsWaypoint()
    {
      var addr = GeocodeAddress();
      if (addr != null)
      {
        this.wpList.Add(new Waypoint
        {
          Latitude = addr.geoPosition.latitude.GetValueOrDefault(),
          Longitude = addr.geoPosition.longitude.GetValueOrDefault(),
          Notify = true,
          Visible = true,
          Description = addr.positionText
        });
        this.bindingSource.ResetBindings(false);
      }
    }

    /// <summary>
    /// Exports the current list of waypoints as ITN file.
    /// </summary>
    private void SaveToItnFile()
    {
      SaveFileDialog dlg = new SaveFileDialog();
      dlg.Title = "Save waypoints to ITN file";
      dlg.DefaultExt = "itn";
      dlg.Filter = "ITN files (*.itn)|*.itn|All files (*.*)|*.*";
      if (DialogResult.OK == dlg.ShowDialog())
      {
        SaveToItnFile(dlg.FileName);
        MessageBox.Show("Saved to ITN file.", "ITN export");
      }
    }

    /// <summary>
    /// Exports the current list of waypoints as ITN file.
    /// </summary>
    private void SaveToItnFile(string fileName)
    {
      StreamWriter sw = new StreamWriter(fileName, false, this.encodingComboBox.SelectedIndex == 0 ? Encoding.Default : Encoding.UTF8);

      foreach (Waypoint wp in this.wpList)
      {
        var line = String.Format("{0:D}|{1:D}|{2}|0|", wp.Longitude / 10, wp.Latitude / 10, wp.Description);
        sw.WriteLine(line);
      }
      sw.Close();
    }

    private void saveItnButton_Click(object sender, EventArgs e)
    {
      SaveToItnFile();
    }

    /// <summary>
    /// Loads the most recent order for the given object. This is used for displaying order details after successfully sending
    /// an order. 
    /// </summary>
    /// <param name="objectNo"></param>
    /// <returns></returns>
    private OrdersService.ReportedOrderData LoadRecentOrder(string objectNo)
    {
      var aParm = new OrdersService.AuthenticationParameters
      {
        accountName = this.accountTextBox.Text,
        userName = this.userTextBox.Text,
        password = this.passwordTextBox.Text,
        apiKey = this.apiKeyTextBox.Text.Length > 0 ? this.apiKeyTextBox.Text : null
      };
      var gParm = new OrdersService.GeneralParameters();
      var client = new OrdersService.ordersClient(SelectedEndpoint.OrdersEndpoint);
      var oParam = new OrdersService.OrderReportParameters();
      oParam.@object = new OrdersService.ObjectIdentityParameter { objectNo = objectNo };
      oParam.dateRange = new OrdersService.DateRange
      {
        rangePattern = OrdersService.DateRangePattern.D0
      };
      var result = client.showOrderReport(aParm, gParm, oParam);
      if (result.statusCode == 0 && result.resultSize > 0)
      {
        var order = result.results[0];
        return order as OrdersService.ReportedOrderData;
      }
      return null;
    }


    /// <summary>
    /// Load the waypoints of an existing order in WEBFLEET.
    /// </summary>
    /// <param name="orderNo"></param>
    /// <returns></returns>
    private List<OrdersService.ItnWaypoint> LoadWaypoints(string orderNo)
    {
      var aParm = new OrdersService.AuthenticationParameters
      {
        accountName = this.accountTextBox.Text,
        userName = this.userTextBox.Text,
        password = this.passwordTextBox.Text,
        apiKey = this.apiKeyTextBox.Text.Length > 0 ? this.apiKeyTextBox.Text : null
      };
      var gParm = new OrdersService.GeneralParameters();
      var client = new OrdersService.ordersClient(SelectedEndpoint.OrdersEndpoint);
      var wpParam = new OrdersService.ShowWaypointsParameter() { orderNo = orderNo };
      var result = client.showOrderWaypoints(aParm, gParm, wpParam);
      List<OrdersService.ItnWaypoint> wpList = new List<OrdersService.ItnWaypoint>();
      if (result.resultSize > 0)
      {
        wpList.AddRange((from OrdersService.ItnWaypoint wp in result.results select wp));
      }
      return wpList;
    }


    private void showOrdersButton_Click(object sender, EventArgs e)
    {
      var client = new OrdersService.ordersClient(SelectedEndpoint.OrdersEndpoint);
      var credentials = new Credentials
      {
        account = this.accountTextBox.Text,
        user = this.userTextBox.Text,
        password = this.passwordTextBox.Text,
        apiKey = this.apiKeyTextBox.Text.Length > 0 ? this.apiKeyTextBox.Text : null
      };
      OrderListDlg dlg = new OrderListDlg(client, credentials);
      if (DialogResult.OK == dlg.ShowDialog())
      {
        var order = dlg.SelectedOrder;
        ClearOrderInputFields();
        FillFieldsFromOrder(order);
      }
      dlg.Dispose();
    }

    private void FillFieldsFromOrder(OrdersService.ReportedOrderData order)
    {
      this.orderNoTextBox.Text = order.orderId;
      this.orderTextTextBox.Text = order.orderText;
      this.objectNoTextBox.Text = order.@object.objectNo;
      this.latLonLabel.Text = order.geoPosition.latitude + " " + order.geoPosition.longitude;
      this.wpList.Clear();
      var wpList = LoadWaypoints(order.orderId);
      this.wpList.AddRange(from OrdersService.ItnWaypoint wp in wpList
                           select
                             new Waypoint
                             {
                               Latitude = wp.latitude,
                               Longitude = wp.longitude,
                               Description = wp.description,
                               Notify = wp.notifyPassing,
                               Visible = wp.showInList
                             });
      this.bindingSource.ResetBindings(false);
    }

    private void ClearOrderInputFields()
    {
      this.orderNoTextBox.Text = "";
      this.orderTextTextBox.Text = "";
      this.objectNoTextBox.Text = "";
      this.addrStreetNoTextBox.Text = "";
      this.addrStreetTextBox.Text = "";
      this.addrTownTextBox.Text = "";
      this.addrZipCodeTextBox.Text = "";
    }

    private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
    {
      SaveState();
    }

    private void SaveState()
    {
      try
      {
        string folder = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + "\\ItnWaypointTool";
        if (!Directory.Exists(folder))
        {
          Directory.CreateDirectory(folder);
        }
        Settings settings = new Settings
        {
          account = this.accountTextBox.Text,
          user = this.userTextBox.Text,
          server = this.endPointComboBox.Text,
          encodingIndex = this.encodingComboBox.SelectedIndex,
          copyrightShown = true,
          apiKey = this.apiKeyTextBox.Text
        };
        string fileName = folder + "\\settings.dat";

        using (FileStream fs = File.Create(fileName))
        {
          BinaryFormatter formatter = new BinaryFormatter();
          formatter.Serialize(fs, settings);
        }
      }
      catch (Exception e)
      {
        // ignore to allow graceful application close
      }
    }

    private void RestoreState()
    {
      string folder = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + "\\ItnWaypointTool";
      string fileName = folder + "\\settings.dat";
      try
      {
        using (FileStream fs = File.OpenRead(fileName))
        {
          BinaryFormatter formatter = new BinaryFormatter();
          Settings settings = (Settings)formatter.Deserialize(fs);
          this.userTextBox.Text = settings.user;
          this.accountTextBox.Text = settings.account;
          this.apiKeyTextBox.Text = settings.apiKey;
          this.endPointComboBox.Text = settings.server;
          this.encodingComboBox.SelectedIndex = settings.encodingIndex;
          this.passwordTextBox.Focus();
          this.mustShowCopyright = !settings.copyrightShown;
        }
      }
      catch (Exception e)
      {
        // ignore. maybe there's no settings file
      }
    }

    private void updateOrderButton_Click(object sender, EventArgs e)
    {
      UpdateOrder();
    }

    private void panel1_MouseClick(object sender, MouseEventArgs e)
    {
      ShowCopyrights();
    }

    private static void ShowCopyrights()
    {
      CopyrightDialog dlg = new CopyrightDialog();
      dlg.ShowDialog();
      dlg.Dispose();
    }

    private void MainForm_Shown(object sender, EventArgs e)
    {
      if (this.mustShowCopyright)
      {
        ShowCopyrights();
      }
    }


  }
}
