﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace ItnWaypoints
{
  public partial class OrderListDlg : Form
  {
    OrdersService.ordersClient client; 
    Credentials credentials;
    List<OrdersService.ReportedOrderData> orderList;
    BindingSource bindingSource = new BindingSource();
    List<OrderEntry> ordersDataSource = new List<OrderEntry>();

    int selectedIndex = 0;

    public OrderListDlg(OrdersService.ordersClient client, Credentials credentials)
    {
      InitializeComponent();

      this.client = client;
      this.credentials = credentials;
      this.bindingSource.DataSource = this.ordersDataSource;
      this.dataGridView.DataSource = this.bindingSource;

      this.dateRangeComboBox.DisplayMember = "Name";
      this.dateRangeComboBox.ValueMember = "Pattern";
      this.dateRangeComboBox.DataSource = new []
      {
        new { Name = "today", Pattern = OrdersService.DateRangePattern.D0 },
        new { Name = "yesterday", Pattern = OrdersService.DateRangePattern.Dm1 },
        new { Name = "this week", Pattern = OrdersService.DateRangePattern.W0 },
        new { Name = "previous week", Pattern = OrdersService.DateRangePattern.Wm1 },
        new { Name = "this month", Pattern = OrdersService.DateRangePattern.M0 },
        new { Name = "previous month", Pattern = OrdersService.DateRangePattern.Mm1 }
      };
      //this.dateRangeComboBox.SelectedItem = this.dateRangeComboBox.Items[0];
      this.dateRangeComboBox.SelectedIndex = 0;

    }

    private void OrderListDlg_Load(object sender, EventArgs e)
    {
      FillOrderList();
    }

    private void FillOrderList()
    {
      this.orderList = LoadOrders();
      if (this.orderList != null)
      {
        this.ordersDataSource.Clear();
        this.ordersDataSource.AddRange(from OrdersService.ReportedOrderData order in this.orderList
                                       select new
                                         OrderEntry
                                       {
                                         orderDate = order.orderDate.Value,
                                         orderNo = order.orderId,
                                         orderText = order.orderText,
                                         objectNo = order.@object.objectNo,
                                         destination = order.destination
                                       });
        this.bindingSource.ResetBindings(false);
      }
    }

    private void okButton_Click(object sender, EventArgs e)
    {
      this.selectedIndex = this.dataGridView.CurrentRow.Index;
    }

    public OrdersService.ReportedOrderData SelectedOrder
    {
      get
      {
        return this.orderList[this.selectedIndex];
      }
    }

    private List<OrdersService.ReportedOrderData> LoadOrders()
    {
      var aParm = new OrdersService.AuthenticationParameters
      {
        accountName = this.credentials.account,
        userName = this.credentials.user,
        password = this.credentials.password,
        apiKey = this.credentials.apiKey
      };
      var gParm = new OrdersService.GeneralParameters();
      var oParam = new OrdersService.OrderReportParameters();
      oParam.@object = new OrdersService.ObjectIdentityParameter();
      oParam.dateRange = new OrdersService.DateRange
      {
        rangePattern = (OrdersService.DateRangePattern) this.dateRangeComboBox.SelectedValue
      };
      var result = client.showOrderReport(aParm, gParm, oParam);
      List<OrdersService.ReportedOrderData> orderList = null;
      if (result.statusCode == 0 || result.statusCode == 9198)
      {
        orderList = new List<OrdersService.ReportedOrderData>();
        if (result.results != null)
        {
          orderList.AddRange((from OrdersService.ReportedOrderData order in result.results select order));
        }
      }
      else
      {
        Tools.DisplayError(result);
      }
      return orderList;
    }

    private void reloadButton_Click(object sender, EventArgs e)
    {
      FillOrderList();
    }

    private void dataGridView_CellContentDoubleClick(object sender, DataGridViewCellEventArgs e)
    {
      this.selectedIndex = this.dataGridView.CurrentRow.Index;
      this.DialogResult = DialogResult.OK;
      Close();
    }


  }
}
